%%%%%%%%%%%%%%%
define get_grid()
%%%%%%%%%%%%%%%
%!%+
%\function{get_grid}
%\usage{(XX, YY) = get_grid(Double_Type X[], Double_Type Y[]);}
%\description
%    \code{XX} and \code{YY} are two-dimensional arrays
%    which span the grid defined by the arrays \code{X} and \code{Y}.
%\example
%    variable XX, YY;  (XX, YY) = get_grid([-2:2:0.01], [-2:2:0.01]);
%    plot_image( atan2(YY, XX) );
%!%-
{
  variable X, Y;
  switch(_NARGS)
  { case 2: (X, Y) = (); X = [X]; Y = [Y]; }
  { help(_function_name()); return; }

  variable iX, nX = length(X);
  variable iY, nY = length(Y);
  variable XX = Double_Type[nY, nX];
  variable YY = Double_Type[nY, nX];
  _for iX (0, nX-1, 1)
    XX[*, iX] = X[iX];
  _for iY (0, nY-1, 1)
    YY[iY, *] = Y[iY];
  return (XX, YY);
}
