%%%%%%%%%%%%%%%%%
define interpol2D()
%%%%%%%%%%%%%%%%%
%!%+
%\function{interpol2D}
%\synopsis{}
%\usage{img2 = interpol2D(X2, Y2,  X, Y, img);}
%\description
%    \code{img = Double_Type[ny, nx];}\n
%    \code{X  = Double_Type[nx];}\n
%    \code{Y  = Double_Type[ny];}\n
%    \code{min(X) <= X2 <= max(X)}\n
%    \code{min(Y) <= Y2 <= max(Y)}\n
%!%-
{
  variable X2, Y2, X, Y, img;
  switch(_NARGS)
  { case 5: (X2, Y2, X, Y, img) = (); }
  { help(_function_name()); return; }

  if(any(X2 < min(X)) or any(X2 > max(X)))
  { vmessage("error (%s): min(X) <= X2 <= max(X) is not fulfilled.", _function_name());
    return;
  }
  if(any(Y2 < min(Y)) or any(Y2 > max(Y)))
  { vmessage("error (%s): min(Y) <= Y2 <= max(Y) is not fulfilled.", _function_name());
    return;
  }

  variable img2 = Double_Type[ length(Y2), length(X2) ];

  _for $1 (0, length(Y2)-1 ){
    variable iy = wherelast( Y[[:-2]] <= Y2[$1] );    % such that  Y[iy] <= Y2[$1] < Y[iy+1]  holds
    
    variable dy  = Y2[$1] - Y[iy];
    variable dy_ = Y[iy+1] - Y2[$1];

    _for $2 (0, length(X2)-1 ){
      variable ix = wherelast( X[[:-2]] <= X2[$2] );   % such that  X[ix] <= X2[$2] < X[ix+1]  holds

      variable dx  = X2[$2] - X[ix];
      variable dx_ = X[ix+1] - X2[$2];

      variable w00 = dy_ * dx_;
      variable w10 = dy  * dx_;
      variable w01 = dy_ * dx;
      variable w11 = dy  * dx;

      img2[$1, $2] = (w00*img[iy,ix] + w10*img[iy+1,ix] + w01*img[iy,ix+1] + w11*img[iy+1,ix+1])
	            /(w00            + w10              + w01              + w11               );
    }
  }

  return img2;
}
