define interpol_2d_table()
%!%+
%\function{interpol_2d_table}
%\usage{Double_Type interpol_2d_table(Double_Type x, y, Struct_Type table)}
%!%-
{
  variable x, y, tab, X, Y, Z;
  switch(_NARGS)
  { case 3: (x, y, tab) = (); }
  { case 5: (x, y, X, Y, Z) = (); tab = struct { x=X, y=Y, z=Z }; }
  { help(_function_name()); }

  if( min(tab.x) <= x <= max(tab.x)  &&  min(tab.y) <= y <= max(tab.y) )
  {
    variable ix, iy;
    for(ix=0; ix<length(tab.x)-2 && (tab.x[ix]-x)*(tab.x[ix+1]-x)>0; )
      ix++;
    for(iy=0; iy<length(tab.y)-2 && (tab.y[iy]-y)*(tab.y[iy+1]-y)>0; )
      iy++;
    if(qualifier_exists("verbose"))
    { vmessage("x[%d] = %f  <=  x = %f  <=  x[%d] = %f", ix, tab.x[ix],  x,  ix+1, tab.x[ix+1]);
      vmessage("y[%d] = %f  <=  y = %f  <=  y[%d] = %f", iy, tab.y[iy],  y,  iy+1, tab.y[iy+1]);
    }
    variable w00 = (tab.x[ix+1]-x) * (tab.y[iy+1]-y);
    variable w01 = (tab.x[ix+1]-x) * (y-tab.y[iy]);
    variable w10 = (x-tab.x[ix]) * (tab.y[iy+1]-y);
    variable w11 = (x-tab.x[ix]) * (y-tab.y[iy]);
    return (w00*tab.z[iy,ix] + w10*tab.z[iy+1,ix] + w01*tab.z[iy,ix+1] + w11*tab.z[iy+1,ix+1])
          /(w00              + w10                + w01                + w11                 );
  }
  vmessage("error (%s): extrapolation not allowed", _function_name());
  return log(-1.);
}
