%%%%%%%%%%%%%%%%%%%%%%
define rectangle_where()
%%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{rectangle_where}
%\synopsis{finds the rectangle in a 2d array, where an expression is true}
%\usage{(Integer_Type Y[], X[]) = rectangle_where(Integer_Type expr);}
%\description
%    The rectangle in the 2d array defined by the index-arrays
%    \code{Y} and \code{X} contains all elements for which \code{expr!=0},
%    i.e., one can crop the elements for which \code{expr==0}.
%    (Note that \code{all(expr[Y,X])} is not necessarily true.)
%\example
%    img = img[rectangle_where(img>0)];
%!%-
{
  variable expr;
  switch(_NARGS)
  { case 1: expr = (); }
  { help(_function_name()); return; }

  variable dims = array_shape(expr);
  variable w=dims[1], h=dims[0];
  variable xmin = 0;    while(xmin<w-1 && all(expr[*,xmin]==0))  xmin++;
  variable xmax = w-1;  while(xmax>0   && all(expr[*,xmax]==0))  xmax--;
  variable ymin = 0;    while(ymin<h-1 && all(expr[ymin,*]==0))  ymin++;
  variable ymax = h-1;  while(ymax>0   && all(expr[ymax,*]==0))  ymax--;

  return [ymin:ymax], [xmin:xmax];
}
