define binary_search()
%!%+
%\function{binary_search}
%\synopsis{searches a value in a sorted list}
%\usage{Integer_Type ndx=binary_search(Array_Type arr, Data_Type val);}
%\qualifiers{
%\qualifier{interval}{do an interval search, return
%   ndx such that arr[ndx]<=val<add[ndx+1], or 0 if val<=arr[0] or
%   length(arr) if add[-1]<=val, see description. }
%}
%\description
%  The function performs a binary search for the value val in the array
%  arr. All data types where the comparison operators work are allowed
%  (i.e., mainly all numerical types).
%
%  Values in arr MUST be unique and sorted. This is not checked.
%  The binary search is much faster than, e.g., a call to
%  where, since the where function is O(n), while binary_search
%  scales is O(lb(N)) where lb is the logarithm to base 2.
%
%  If no qualifiers are set, the function returns the index of the array
%  element if val is found. The function returns -1 if the value is not found.
%
%  If the interval qualifier is set, then the function will do an interval
%  search such that arr[ndx]<=value<arr[ndx]. The function will return
%  0 if value<=arr[0] and length(arr) if value>arr[-1]. This can be
%  used to search the index where one would insert a number into
%  an array.
%!%-
{
    variable arr;
    variable value;

    (arr,value)=();

    variable left,mid,right;
    
    
    if (qualifier_exists("interval")) {
        % returns ndx such that
        % arr[ndx]<=val<arr[ndx+1]
        variable ll=length(arr);

        left=0;
        right=ll;

        while (left<right) {
            mid=(left+right)/2;
            if (arr[mid]==value) {
                return mid;
            } else {
                if (arr[mid]<value) {
                    left=mid+1;
                } else {
                    right=mid-1;
                }
            }
        }
        if (left==ll) {
            return ll;
        }
        if (arr[left]>value) {
            return left;
        }
        return left+1;
    }
    
    % returns ndx such that arr[ndx]==val
    left=0;
    right=length(arr)-1;

    while (left<=right) {
        mid=(left+right)/2;
        if (arr[mid]<value) {
            left=mid+1;
        } else {
            if (arr[mid]>value) {
                right=mid-1;
            } else {
                return mid;
            }
        }
    }
    return -1;
}
