define struct_of_arrays_2_struct_array()
%!%+
%\function{struct_of_arrays_2_struct_array}
%\synopsis{Converts a Struct_Type with array fields to a Struct_Type[]}
%\usage{Struct_Type[] struct_of_arrays_2_struct_array(Struct_Type S);}
%\description
%   This function converts a 'Struct_Type' containing fields
%   which are arrays (all arrays must have the same dimension!)
%   to a Struct-array 'Struct_Type[]' with the same fields but
%   one dimensional. That means the returned Struct_Type[] has
%   then the dimension (array_shape) of the fields.
%   The returned Struct-Array will contain all fields the original
%   'S' also had, BUT for those fields with a different
%   array_shape then the first field and the value will be set to the
%   first entry of its field!
%\example
%   variable s = struct{ a=[1:3], b=[4:6], c=[3,20] };
%   variable sa = struct_of_arrays_2_struct_array( s );
%   print(sa);
%   {a=1, b=4, c=3}
%   {a=2, b=5, c=3}
%   {a=3, b=6, c=3}
%\seealso{array_map}
%!%-
{
  variable s;
  switch(_NARGS)
  { case 1: s = (); }
  { help(_function_name); }

  %%% Get the field names
  variable fnames = get_struct_field_names( s );
  variable fields = get_struct_fields( s, fnames );

  %%% Create a Struct-Array with the dimensions of the first field
  variable type = typeof(s);
  variable n = length(fields[0]);

  variable sa;
  if( type == Struct_Type ){
    sa = struct_array( n, @type(fnames) );
  }
  else{
    sa = @type[n];
  }
  
  variable i;
  _for i ( 0, length(fnames)-1 ){
    array_map( Void_Type, &set_struct_field, sa, fnames[i],  fields[i] );
  }
  return sa;
}
