define weighted_mean()
%!%+
%\function{weighted_mean}
%\usage{Double_Type m = weighted_mean(x, w);
%\altusage{Double_Type m = weighted_mean(x; err=err);  % or weighted_mean(x, err; err)}
%}
%\description
%    \code{m = sum(w*x) / sum(w);}\n
%    \code{%} for Gaussian errors:\n
%    \code{w = 1/err^2;}
%!%-
{
  variable x, w=NULL, err=qualifier("err", NULL);
  switch(_NARGS)
  { case 1: x = (); }
  { case 2:
      if(qualifier_exists("err"))
      { if(err!=NULL)  vmessage("warning (%s): too many arguments -- qualifier-value for err ignored", _function_name()); 
        (x, err) = ();
      }
      else
        (x, w) = ();
  }
  { help(_function_name()); return; }

  if(err!=NULL)  w = 1/err^2;
  if(w==NULL)  w = 0*x + 1;

  return sum(w*x)/sum(w);
}
