%%%%%%%%%%%%%%%%%%%%%%
define Roche_potential()
%%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{Roche_potential}
%\usage{Double_Type Roche_potential(Double_Type x[, y[, z]]);}
%\qualifiers{
%\qualifier{q}{mass ratio}
%\qualifier{sph}{coordinates are considered as spherical coordinates r, phi, theta}
%}
%\description
%   The default values for y and z is 0.
%
%   The Roche potential has the following form:
%   \code{-1/sqrt(x^2 + y^2 + z^2 ) - q/sqrt( (x-1.)^2 + y^2 + z^2 ) - (1+q)/2.*( (x-q/(1.+q))^2 + y^2 )}
%!%-
{
  variable x=0, y=0, z=0;
  switch(_NARGS)
  { case 1:  x        = (); }
  { case 2: (x, y)    = (); }
  { case 3: (x, y, z) = (); }
  { help(_function_name()); return; }

  variable q = qualifier("q", .567);
%  if(qualifier_exists("verbose"))
%    vmessage("q=%f", q);
  if (qualifier_exists("sph"))
  {
    variable siny, cosy, sinz = sin(z);
    (siny, cosy) = sincos(y);

    return -1./x - q/sqrt(x^2 - 2.*x*cosy*sinz + 1) - (1+q)/2.*( (x*sinz*cosy-q/(1.+q))^2 + (x*sinz*siny)^2 );
  }
  else return - 1./sqrt(x^2 + y^2 + z^2 ) - q/sqrt( (x-1.)^2 + y^2 + z^2 ) - (1+q)/2.*( (x-q/(1.+q))^2 + y^2 );
}
