define frequency2velocity()
%!%+
%\function{frequency2velocity}
%\synopsis{Calculate the velocity to a given frequency.}
%\usage{frequency2velocity(freq,restfreq);}
%\qualifiers{
%    \qualifier{veldef}{The velocity definition which must be one of
%                   OPTICAL, RADIO, or TRUE.  Defaults to RADIO.}
%}
%\description
%    Convert frequency to velocity (m/s) using the given rest
%    frequency and velocity definition. The units (Hz, MHz, GHz, etc)
%    of the frequencies to convert must match that of the rest frequency argument.
%    
%    Adopted from the gbtidl script freqtovelo.pro
%!%-
{
   variable freq, restfreq;
   switch(_NARGS)
     { case 2: (freq,restfreq) = (); }
     { help(_function_name()); return; }

   variable veldef = qualifier("veldef","RADIO");
   switch(veldef)
     {
      case "RADIO": variable vel = Const_c*1e-2*(1. - freq / restfreq);
     }
     {
      case "OPTICAL": vel = Const_c*1e-2*(restfreq / freq -1.);
     }
     {
      case "TRUE": variable g = (freq / restfreq)^2;
	vel = Const_c*1e-2*(1. - g) / (1. + g);
     }
     {
       message( "unrecognized velocity definition");
     } 

    return vel;
}
