%%%%%%%%%%%%%%
define zang ()
%%%%%%%%%%%%%%
%!%+
%\function{zang}
%\synopsis{Determine the angular size of an object as a function of redshift}
%\usage{Double_Type angsize = zang (Double_Type size, Double_Type redshift);}
%\description
%    Requires an input size in kpc and returns an angular size
%    in arc seconds. (Analog to the function \code{zsize},
%    which converts angular size to projected size size given the
%    redshift.)
%    The default cosmology according is used. The function passes
%    qualifiers to the functions \code{lumdist} and \code{cosmo_param} allowing
%    to change the cosmology.
%\example
%    variable size = 50.; % kpc
%    variable    z = 1.5; % redshift
%    zang(50,1.5; omega_m = 0.3, omega_lambda = 0.0, silent);
%    % ---> 6.58 arc seconds
%\seealso{lumdist, cosmo_param}
%!%-
{
  variable dl, z;

  switch(_NARGS)
  { case 2: (dl,z) = ();}
  { help(_function_name()); return; }
  
  variable d =  lumdist(z;; __qualifiers);

  % Angular distance is equal to the luminosity distance times (1+z)^2
  return 180./PI*3600.*dl*(1.+z)^2/(1000.*d);
}
