define zsize ()
%!%+
%\function{zsize}
%\synopsis{Determine the projected size of an object as a function of redshift}
%\usage{Double_Type proj_size = zsize (Double_Type angle, Double_Type redshift);}
%\description
%    Requires an input angular size in arc seconds and returns
%    the projected size in kpc. (Analog to the function \code{zang},
%    which converts projected size to angular size given the
%    redshift.)
%    The default cosmology according is used. The function passes
%    qualifiers to the functions \code{lumdist} and \code{cosmo_param} allowing
%    to change the cosmology.
%\example
%    variable angsize = 1.; % arc seconds
%    variable    z = 1.5; % redshift
%    zsize(angsize,z);
%    % ---> 8.46 kpc
%\seealso{zang, lumdist, cosmo_param}
%!%-
{
  variable ang, z;

  switch(_NARGS)
  { case 2: (ang,z) = ();}
  { help(_function_name()); return; }
  
  variable d =  lumdist(z;; __qualifiers);

  % Angular distance is equal to the luminosity distance times (1+z)^2
  return ang*PI/(180.0*3600)*(1.0+z)^(-2) * 1000.0*d;
}
