private define qph6_to_col(q, p, h6)
{      if(h6<1.) return p + (q-p)*h6;
  else if(h6<3.) return q;
  else if(h6<4.) return p + (q-p)*(4.-h6);
                 return p;
}

define hsl2rgb()
%!%+
%\function{hsl2rgb}
%\synopsis{converts a (hue, saturation, lightness) color to (red, green, blue)}
%\usage{Integer_Type rgb = hsl2rgb(Double_Type h, s, l);}
%\description
%    The hue \code{h} is an angle in color-space. Here, \code{h} is between 0 and 1.
%    \code{h} of red is 0, \code{h} of green is 1/3, and \code{h} of blue is 2/3.
%    The lightness \code{l} of black is 0, and \code{l} of white is 1.
%    Like the hue, the saturation \code{0<=s<=1} matters only for \code{0<l<1}.
%
%    The return value is a(n array of) 24 bit RGB value(s).
%\seealso{rgb2hsl}
%!%-
{
  variable h, s, l, r, g, b;
  switch(_NARGS)
  { case 3: (h, s, l) = (); }
  { return help(_function_name()); }

  if(typeof(h)==Array_Type)
    return array_map(Integer_Type, &hsl2rgb, h, s, l);
% {
%   variable rgb = array_map(Integer_Type, &hsl2rgb, h, s, l);
%   if(qualifier_exists("r_g_b"))
%     return (rgb & 0xFF0000) >> 16, (rgb & 0xFF00) >> 8, rgb & 0xFF;  % in inverval [0, 255]
%   else
%    return rgb;
% }

  if(s==0)
    (r, g, b) = (l, l, l);
  else
  { variable q = l*(1.+s);   if(l>0.5)  q = l + s - l*s;
    variable p = 2.*l - q;
    r = qph6_to_col(q, p, (h*6+2.) mod 6.);
    g = qph6_to_col(q, p,  h*6           );
    b = qph6_to_col(q, p, (h*6+4.) mod 6.);
  }

% if(qualifier_exists("r_g_b"))
%   return r, g, b;   % in interval [0, 1]
% else
    return   ((typecast(255.9999999999999*r, UChar_Type) & 0xFF) shl 16)
           + ((typecast(255.9999999999999*g, UChar_Type) & 0xFF) shl 8)
	   +  (typecast(255.9999999999999*b, UChar_Type) & 0xFF);
}
