%%%%%%%%%%%%%%%%%%
define fermi2MJD()
%%%%%%%%%%%%%%%%%%
%!%+
%\function{fermi2MJD}
%\synopsis{calculate MJD from Fermi seconds}
%\usage{Double_Type = fermi2MJD (Double_Type);}
%\description
%	Calculates MJD(UTC) for a given Fermi Mission
%	Elapsed Time (MET) in seconds.
%\example
%	variable m = 239557420.0;
%	variable fermi_s = fermi2MJD(m);
%\seealso{MJD2fermi, MJDref_satellite}
%!%-
{ 
  variable fermi_in;
  
  switch(_NARGS)
  { case 1: fermi_in  = (); }
  { help(_function_name()); return; }

  variable mjd_ref=51910.; % MJDref_satellite ("fermi");
  variable diff1=utc2tai(mjd_ref;mjd,deltat);
  variable diff2=utc2tai(mjd_ref+fermi_in/86400.;mjd,deltat);
  return mjd_ref + (fermi_in-(diff2-diff1))/86400.;
}
