define time_array( time )
%!%+
%\function{time_array}
%\synopsis{Converts a time [s] (e.g., _time) into an array [sec,min, ...]}
%\usage{Integer_Type[] = time_array( Integer_Type );}
%\altusage{String_Type = time_array( Integer_Type ; str);}
%\qualifiers{
%\qualifier{str}{Instead of a time array a String_Type is returned!}
%}
%\description
%   Converts a time given in seconds into a time array, where
%   the first entry corresponds to the seconds, the second to
%   the minutes, etc. The length of the array is variable,
%   i.e., if the given time is of the order of hours, the array
%   will not include days and years.
%
%   If the qualifier 'str' is given the array will be converted
%   into a string (see example).
%\example
%   isis> ta = format_time ( _time ); ta;
%   Integer_Type[5];
%   isis> print(ta);
%   54
%   1
%   14
%   302
%   43
%
%   %or
%   isis> ts = format_time ( _time ;str ); ts;
%   43y 302d 14h  1m 54s
%   
%\seealso{_time}
%!%-
{
  variable ftime = Integer_Type[0];
  variable tmod = [ 60, 60, 24, 365.25 ];

  variable rest = @time;
  variable i;
  for( i=0 ; i<length(tmod) and rest!=0 ; ++i ){
    ftime = [ ftime, rest mod tmod[i] ];
    rest -= ftime[-1];
    rest /= tmod[i];
    }
  if( rest != 0 )
    ftime = [ ftime, rest ];
  ftime = int(ftime);

  if( qualifier_exists("str") ){
    variable sftime="";
    variable unit = ["s","m","h","d","y"];
    _for i ( length(ftime)-1, 0, -1 ){
      sftime += sprintf("%2d%s ",ftime[i],unit[i]);
	  }
    return sftime[[:-2]];
    }
  return ftime;
}
