define tt2tai()
%!%+
%\function{tt2tai}
%\synopsis{Convert a (M)JD in Terrestrial Time (TT, aka TDT or ET) into TAI}
%\usage{tai=tt2tai(JD)}
%\description
%This routine converts a Julian Date that is given in Terrestrial Time
%(formerly known as Terrestrial Dynamic Time or Ephemeris Time) into
%TAI. This is done by subtracting 32.184s from the TT.
% This routine is array safe.
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%\qualifier{deltat}{Return the TAI-TT in seconds, rather than the corrected (M)JD.}
%}
%\seealso{utc2tai,tai2utc,tai2tt,tt2tdb}
%!%-
{
    variable JD;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {
	help(_function_name()); return;
    }

    variable diff=-32.184;

    if (qualifier_exists("deltat")) {
	if (typeof(JD)==Array_Type) {
            return diff+Double_Type[length(JD)];
	}
	return diff;
    }
    return JD+diff/86400.0;
}
